//
// $Id: Reader.hpp 2027 2010-06-09 22:54:33Z chambm $
//
//
// Original author: Matt Chambers <matt.chambers .@. vanderbilt.edu>
//
// Copyright 2008 Spielberg Family Center for Applied Proteomics
//   Cedars Sinai Medical Center, Los Angeles, California  90048
// Copyright 2008 Vanderbilt University - Nashville, TN 37232
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//


#ifndef _READER_HPP_CLI_
#define _READER_HPP_CLI_


#pragma warning( push )
#pragma warning( disable : 4634 4635 )
#include "MSData.hpp"
#include "pwiz/data/msdata/Reader.hpp"
#pragma warning( pop )


namespace pwiz {
namespace CLI {
namespace msdata {


public DEFINE_STD_VECTOR_WRAPPER_FOR_REFERENCE_TYPE(MSDataList, pwiz::msdata::MSDataPtr, MSData, NATIVE_SHARED_PTR_TO_CLI, CLI_TO_NATIVE_SHARED_PTR);


/// interface for file readers
public ref class Reader
{
    DEFINE_INTERNAL_BASE_CODE(Reader, pwiz::msdata::Reader);

    public:

    /// return true iff Reader can handle the file;
    /// Reader may filter based on filename and/or head of the file
    virtual bool accept(System::String^ filename,
                        System::String^ head);

    /// fill in the MSData structure
    virtual void read(System::String^ filename,
                      System::String^ head,
                      MSData^ result);

    /// fill in the MSData structure
    virtual void read(System::String^ filename,
                      System::String^ head,
                      MSData^ result,
                      int runIndex);

    /// fill in the MSData structure
    virtual void read(System::String^ filename,
                      System::String^ head,
                      MSDataList^ results);

    /// fill in the MSData structure
    virtual array<System::String^>^ readIds(System::String^ filename,
                                            System::String^ head);
};


/// Reader container (composite pattern).
///
/// The template get&lt;reader_type>() gives access to child Readers by type, to facilitate
/// Reader-specific configuration at runtime.
///
public ref class ReaderList : public Reader
{
    DEFINE_DERIVED_INTERNAL_BASE_CODE(pwiz::msdata, ReaderList, Reader);

    public:

    /// returns child name iff some child identifies, else empty string
	virtual System::String^ identify(System::String^ filename);

    /// returns child name iff some child identifies, else empty string
    virtual System::String^ identify(System::String^ filename,
                                     System::String^ head);

    /// delegates to first child that accepts
    virtual void read(System::String^ filename,
                      MSData^ result);

    /// delegates to first child that accepts
    virtual void read(System::String^ filename,
                      MSData^ result,
                      int runIndex);

    /// delegates to first child that accepts
    virtual void read(System::String^ filename,
                      System::String^ head,
                      MSData^ result) override;

    /// delegates to first child that accepts
    virtual void read(System::String^ filename,
                      System::String^ head,
                      MSData^ result,
                      int runIndex) override;

    /// fill in the MSDataList with MSData for all samples
    virtual void read(System::String^ filename,
                      MSDataList^ results);

    /// fill in the MSDataList with MSData for all samples
    virtual void read(System::String^ filename,
                      System::String^ head,
                      MSDataList^ results) override;

    /// get MSData.Ids
    virtual array<System::String^>^ readIds(System::String^ filename);

    /// get MSData.Ids
    virtual array<System::String^>^ readIds(System::String^ filename,
                                            System::String^ head) override;

    static property ReaderList^ FullReaderList { ReaderList^ get(); }
};


} // namespace msdata
} // namespace CLI
} // namespace pwiz


#endif // _READER_HPP_CLI_
